@@ -4,7 +4,7 @@
 
 /* Copyright Philip Hazel 2021 */
 /* This file created: August 2021 */
-/* This file last modified: June 2025 */
+/* This file last modified: August 2025 */
 
 #include "pmw.h"
 
@@ -42,6 +42,7 @@ static BOOL      midi_onebar_only = FALSE;
 static uint64_t  midi_staves = ~0uL;
 static uint32_t  midi_tempo;
 static int8_t    midi_transpose[MAX_STAVE+1];  /* NB signed */
+static uint8_t   midi_tremolo[MAX_STAVE+1];
 static int       midi_volume = 127;
 static int16_t   next_event_seq;
 static uint32_t  repeat_bar;
@@ -310,7 +311,7 @@ for (stave = 1; stave <= midi_movt->laststave; stave++)
   BOOL noteson;   /* See above comment */
   int32_t moff;
   int midi_stave_status, midi_stave_pitch, midi_stave_velocity;
-  int miditranspose, adjustlength, scrubtremolo;
+  int miditranspose, adjustlength, scrubtremolo, scrubtrill;
 
   if (mac_notbit(midi_staves, stave)) continue;
 
@@ -318,6 +319,7 @@ for (stave = 1; stave <= midi_movt->laststave; stave++)
   miditranspose = midi_transpose[stave];
   adjustlength = 0;
   scrubtremolo = -1;
+  scrubtrill = -1;
   noteson = TRUE;
 
   /* Set up midi parameters */
@@ -456,6 +458,8 @@ for (stave = 1; stave <= midi_movt->laststave; stave++)
         if (change->note < 128)
           midi_stave_pitch = midi_note[stave] = change->note;
 
+        if (change->tremolo < 128) midi_tremolo[stave] = change->tremolo;
+
         /* A voice change must be scheduled to occur in the correct
         sequence with the notes. */
 
@@ -474,10 +478,22 @@ for (stave = 1; stave <= midi_movt->laststave; stave++)
       case b_ornament:
         {
         b_ornamentstr *orn = (b_ornamentstr *)p;
-        if (orn->ornament == or_trem1 ||
-            orn->ornament == or_trem2 ||
-            orn->ornament == or_trem3)
-          scrubtremolo = orn->ornament;
+        switch(orn->ornament)
+          {
+          case or_trem1:
+          case or_trem2:
+          case or_trem3:
+          if ((midi_tremolo[stave] & mtf_repeat) != 0)
+            scrubtremolo = orn->ornament;
+          break;
+
+          case or_tr:
+          case or_trsh:
+          case or_trfl:
+          case or_trnat:
+          scrubtrill = orn->troffset;
+          break;
+          }
         }
       break;
 
@@ -556,7 +572,9 @@ for (stave = 1; stave <= midi_movt->laststave; stave++)
             case (len_quaver*3)/2:    tremolo_multiplier = 3; break;
             }
 
-          if (note->type == b_tremolo && tremolo_multiplier != 0 &&
+          if (note->type == b_tremolo &&
+              (midi_tremolo[stave] & mtf_trill) != 0 &&
+              tremolo_multiplier != 0 &&
               note->next != NULL && note->next->type == b_note &&
               (int)((b_notestr *)note->next)->length == length &&
               scrubtremolo < 0)
@@ -579,10 +597,22 @@ for (stave = 1; stave <= midi_movt->laststave; stave++)
             while (note->type == b_chord);
             }
 
-          /* If the note is followed by a tie, find the next note or chord on
-          the stave. If any of its notes have the same pitch as any of those in
-          the list, extend their playing times. If there are any new notes, add
-          them to the list, with a later starting time. We have to do this
+          /* Else if a single note has a trill ornament, we can set it up in a
+          similar way to tremolo above. */
+
+          else if (pitchcount == 1 && scrubtrill > 0)
+            {
+            scrub = 3 * tremolo_multiplier;
+            pitchlist[1] = pitchlist[0] + scrubtrill;
+            pitchlen[1] = length;
+            pitchstart[1] = nstart - length;
+            alternate_pitchcount++;
+            }
+
+          /* Else if the note is followed by a tie, find the next note or chord
+          on the stave. If any of its notes have the same pitch as any of those
+          in the list, extend their playing times. If there are any new notes,
+          add them to the list, with a later starting time. We have to do this
           because all the notes we are accumulating will be output at the end
           of this bar. Set the noplay flag in the next notes, to stop them
           playing again later. Continue for multiple ties. */
@@ -721,6 +751,7 @@ for (stave = 1; stave <= midi_movt->laststave; stave++)
         }
 
       scrubtremolo = -1;
+      scrubtrill = -1;
       break;
       }  /* End switch on bar item */
     }    /* End of bar scan */
@@ -823,14 +854,14 @@ TRACE("midi_write() movement %d\n", midi_movement);
 
 if (midi_movement < 1 || (usint)midi_movement > movement_count)
   {
-  error(ERR189, midi_movement);
+  error(ERR189, midi_movement, "MIDI");
   return;
   }
 
 midi_movt = movements[midi_movement - 1];
 if (midi_movt->barcount < 1)
   {
-  error(ERR160, midi_movt->number);
+  error(ERR160, midi_movt->number, "MIDI");
   return;
   }
 midi_tempo = midi_movt->miditempo;   /* Default tempo */
@@ -885,10 +916,13 @@ of changing this. */
 
 midi_staves = midi_movt->select_staves;
 
-/* Initialize the tie information */
+/* Initialize the tie and tremolo information */
 
 for (int stave = 1; stave <= midi_movt->laststave; stave++)
+  {
   stavetie[stave] = FALSE;
+  midi_tremolo[stave] = midi_movt->miditremolo;
+  }
 
 /* Miscellaneous stuff */
 
@@ -983,6 +1017,7 @@ for (midi_bar = 0; midi_bar < midi_startbar; midi_bar++)
       b_midichangestr *change;
 
       if (p->type != b_midichange) continue;
+
       change = (b_midichangestr *)p;
       midi_transpose[stave] += change->transpose;
 
@@ -1003,6 +1038,7 @@ for (midi_bar = 0; midi_bar < midi_startbar; midi_bar++)
         }
 
       if (change->note < 128) midi_note[stave] = change->note;
+      if (change->tremolo < 128) midi_tremolo[stave] = change->tremolo;
 
       if (change->voice < 128)
         {
@@ -1010,9 +1046,9 @@ for (midi_bar = 0; midi_bar < midi_startbar; midi_bar++)
         writebyte(0xC0 + midi_channel[stave] - 1);
         writebyte(change->voice);
         }
-      }
-    }
-  }
+      }   /* Loop through bar items on one stave */
+    }     /* Loop through staves */
+  }       /* Loop through bars */
 
 /* Now write the bars */
 
