@@ -2,9 +2,9 @@
 *        PMW native input reading functions     *
 *************************************************/
 
-/* Copyright Philip Hazel 2022 */
+/* Copyright Philip Hazel 2025 */
 /* This file created: March 2021 */
-/* This file last modified: April 2025 */
+/* This file last modified: December 2025 */
 
 /* This file contains the code for reading one note in PMW notation. */
 
@@ -29,6 +29,12 @@ static uint16_t tuckoffset[] = {
   100, 16, 100, 100, 12, 12, 12, 12,
   100, 12, 100, 100, 12, 12, 12, 12 };  /* When bottom is a flat */
 
+/* Numerator and denominator values for multiplying note lengths according to
+the number of dots. The values for offset zero are for the .+ case. */
+
+static int dotnums[] = { 5, 3, 7, 15, 31 };
+static int dotdens[] = { 4, 2, 4, 8, 16 };
+
 /* A number of static variables are used for communication between read_note()
 and post_note(). */
 
@@ -201,11 +207,16 @@ while (p != NULL)
   p->halfway = 0;
   p->offset = 0;
 
-  /* If we are at an equals sign, just output the one equals character;
-  otherwise search for the end of the syllable. */
+  /* If we are at an equals sign, just output the one equals character and
+  remember we've had this case (for XML output); otherwise search for the end
+  of the syllable. */
 
-  if (PCHAR(*s) == '=') s++;
-    else while(Ustrchr("- =", PCHAR(*s)) == NULL) s++;
+  if (PCHAR(*s) == '=')
+    {
+    s++;
+    st->hadlayequals = TRUE;
+    }
+  else while(Ustrchr("- =", PCHAR(*s)) == NULL) s++;
 
   /* Set string count - don't include a minus sign, but skip over it */
 
@@ -262,7 +273,7 @@ void
 read_sortchord(b_notestr *w, uint32_t upflag)
 {
 uint8_t masq = w->masq;                 /* Masquerade value */
-uint32_t dotflags = w->flags & (nf_dot|nf_dot2|nf_plus);  /* Extension flags */
+uint8_t dots = w->dots;
 uint32_t dynamics = 0;                  /* Collected dynamics flags */
 uint32_t fuq = 0;                       /* Free upstemmed quaver flag */
 int p, pp;                              /* Working indices */
@@ -285,14 +296,13 @@ using a simple insertion (there won't be many of them). At the same time, sort
 the current tiedata values into a temporary vector, in the same order, in case
 we need this later. Collect the dynamics and fuq flags as we go (to be put on
 the sorted first note), and set the stem direction flag on each note. Also, set
-the flags for extension dots the same as on the first note. They might be
-different if the first note is masqueraded. Set the same masquerade on all
-notes too. */
+augmentation dots the same as on the first note. They might be different if the
+first note is masqueraded. Set the same masquerade on all notes too. */
 
 while (ww != NULL && ww->type == b_chord)
   {
   uint16_t pitch = ww->spitch;
-  uint32_t flags = (ww->flags & ~(nf_dot|nf_dot2|nf_plus)) | dotflags;
+  uint32_t flags = ww->flags;
   uint32_t acflags = ww->acflags;
 
   if (ww->acc != ac_no) acc_count++;
@@ -306,6 +316,7 @@ while (ww != NULL && ww->type == b_chord)
   ww->flags = upflag | (flags & ~(nf_dotright | nf_invert | nf_stemup));
   ww->acflags = acflags & ~(af_accents | af_opposite);
   ww->masq = masq;
+  ww->dots = dots;
 
   /* Find where to insert this note's pointer. */
 
@@ -386,7 +397,7 @@ while (p < sorttop - 1)
   stem is down. This ensures that the note at the end of the stem is on the
   normal side of the stem. */
 
-  if (upflag)
+  if (upflag != 0)
     {
     increment = 2;
     lowp = p;
@@ -413,7 +424,7 @@ while (p < sorttop - 1)
 
     if (wwH != NULL)
       {
-      if (upflag) wwH->flags |= nf_invert;
+      if (upflag != 0) wwH->flags |= nf_invert;
       if ((wwH->flags & nf_highdot) == 0)
         {
         wwH->flags &= ~nf_lowdot;
@@ -425,7 +436,7 @@ while (p < sorttop - 1)
     accidental, so as to cause accidental positioning to happen if there is
     at least one actual accidental on the chord. */
 
-    if (wwL != NULL && !upflag)
+    if (wwL != NULL && upflag == 0)
       {
       wwL->flags |= nf_invert;
       acc_count++;
@@ -446,9 +457,10 @@ while (p < sorttop - 1)
   }
 
 /* If the stem is up and there are any seconds in the chord, flag all the notes
-to print with any dots moved right. */
+to print with any dots moved right, but when operating right-to-left, it must
+happen for stem-down notes. */
 
-if (SecondsExist && upflag)
+if (SecondsExist && ((upflag != 0) != main_righttoleft))
   for (p = 0; p < sorttop; p++) sortvec[p]->flags |= nf_dotright;
 
 /* Scan the chord to arrange the positioning of the accidentals. This is done
@@ -480,7 +492,7 @@ if (!acc_explicit && acc_count > 1)
     if ((flags & nf_couple) != 0)
       row->pitch += ((flags & nf_coupleU) != 0)? 32 : -32;
 
-    row->inverted = !upflag && ((flags & nf_invert) != 0);
+    row->inverted = upflag == 0 && ((flags & nf_invert) != 0);
     row->acc = ww->acc;
 
     row->accleft = row->orig_accleft = ww->accleft;
@@ -747,7 +759,7 @@ if (!acc_explicit && acc_count > 1)
 descending for stem up and ascending for stem down. If we are dealing with the
 last-read chord, also adjust the order of the read_tiedata vector. */
 
-if (upflag)
+if (upflag != 0)
   {
   p = sorttop - 1;
   pp = 0;
@@ -1202,9 +1214,14 @@ if (!pletending && read_c == '}')
 
 if (pletending)
   {
-  if (brs.pletlen == 0) error(ERR126); else
+  if (brs.pletlen == 0) error(ERR126, "closing curly bracket"); else
     {
-    brs.pletlen = 0;
+    if (pletstackcount > 0)
+      {
+      brs.pletlen = pletstack[--pletstackcount];
+      brs.pletnum = pletstack[--pletstackcount];
+      }
+    else brs.pletlen = 0;
     (void)mem_get_item(sizeof(bstr), b_endplet);
     }
   }
@@ -1313,6 +1330,7 @@ if (read_c == 'x')
 
   pn_notelength = srs.lastnoteptr->length;
   pn_notetype = srs.lastnoteptr->notetype;
+  pn_stemforce = srs.laststemforce;
 
   /* Replicate a note or chord as many times as required. We have to reset the
   relevant pointers each time because they are advanced through a chord. */
@@ -1413,10 +1431,12 @@ for (;;)
   uint32_t flags = srs.noteflags | pn_inchord;
   uint32_t explicit_couple = 0;
 
-  int16_t pitch, abspitch;
+  int16_t pitch, abspitch, pitch_orig;
 
   uint8_t *acc_above = NULL;
   uint8_t acc;
+  uint8_t dots = 0;
+  uint8_t dots_orig = 0;
   uint8_t acc_orig, char_orig;
   uint8_t transposedacc = ac_no;
   uint8_t noteheadstyle = srs.noteheadstyle;
@@ -1573,14 +1593,14 @@ for (;;)
       }
     }
 
-  /* Deal with non-rests, i.e. notes. Get an absolute pitch, independent of any
-  clef. The units are quartertones and the origin is such that middle C has the
-  value 96 (defined as MIDDLE_C), which should cope with all requirements. This
-  pitch doesn't yet contain the accidental (if any). */
+  /* Deal with non-rests, i.e. notes. Get a pitch independent of any clef. The
+  units are quartertones and the origin is such that middle C has the value 96
+  (defined as MIDDLE_C), which should cope with all requirements. This pitch
+  doesn't yet contain the accidental (if any). */
 
   if ('a' <= read_c && read_c <= 'g')
     {
-    pitch = srs.octave + 3*OCTAVE + read_basicpitch[read_c - 'a'];
+    pitch_orig = pitch = srs.octave + 3*OCTAVE + read_basicpitch[read_c - 'a'];
     yextra = srs.stemlength;     /* Default stem length */
     if (!duplicating) pn_onlytieddup = FALSE;
     }
@@ -1591,7 +1611,7 @@ for (;;)
 
   else
     {
-    pitch = 0;
+    pitch_orig = pitch = 0;
     acflags &= ~af_accents;         /* Ignore any default dynamics */
     yextra = srs.rlevel;
     if (read_c == 'q') flags |= nf_hidden;
@@ -1766,7 +1786,7 @@ for (;;)
   /* Deal with non-standard lengths. */
 
   if (brs.pletlen != 0)
-    pn_notelength = (pn_notelength*brs.pletnum)/(brs.pletlen*brs.pletden);
+    pn_notelength = (pn_notelength*brs.pletnum)/brs.pletlen;
 
   /* Deal with dotted notes. For the first note of a chord, a movement of the
   dot is permitted. */
@@ -1792,28 +1812,52 @@ for (;;)
     else error(ERR116, "augmentation dot movement");
     }
 
-  /* Lengthen the note according to .+ . or .. */
+  /* Lengthen the note according to .+ . or .. and save additional flags in
+  dot_orig, for the benefit of MusicXML output. */
+
+  /* Handle augmentation dots, keeping the original in dots_orig for the
+  benefit of MusicXML output, because masquerading may change the basic
+  setting. Up to 4 dots are permitted (see Schumann string quartet Op. 44 #1),
+  and we also support Emmanuel Ghent's '+' notation for extending by a quarter.
+  There are limits on the number of dots for notes shorter than a semiquaver
+  because their lengths don't divide enough. */
 
   if (read_c == '.' )
     {
-    read_nextc();
-    if (read_c == '+')
+    for (int i = 3; i > 0; i--)
       {
-      read_nextc();
-      flags |= nf_plus;
-      pn_notelength = (pn_notelength*5)/4;
-      }
-    else
-      {
-      flags |= nf_dot;
-      pn_notelength = (pn_notelength*3)/2;
-      if (read_c == '.' )
+      if (Ustrncmp(main_readbuffer + read_i, "...", i) == 0)
         {
-        read_nextc();
-        flags |= nf_dot2;
-        pn_notelength = (pn_notelength*7)/6;
+        dots = i + 1;
+        read_i += i;
+        break;
         }
       }
+
+    if ((pn_notetype == dsquaver && dots > 3) ||
+        (pn_notetype == hdsquaver && dots > 2))
+      {
+      error(ERR199);
+      brs.checklength = FALSE;
+      }
+
+    /* If only one dot, look for '.+' and leave temporarily leave dots at zero
+    for that; otherwise we have just one dot. */
+
+    if (dots == 0)
+      {
+      if (main_readbuffer[read_i] == '+') read_i++;
+        else dots = 1;
+      }
+
+    read_nextc();  /* Next character becomes current. */
+
+    /* Adjust the notelength. Then set dots to 255 for .+ and save this
+    original dots value. */
+
+    pn_notelength = (pn_notelength * dotnums[dots])/dotdens[dots];
+    if (dots == 0) dots = 255;
+    dots_orig = dots;
     }
 
   /* Handle doubled or halved note lengths - note that full bar rests must be
@@ -1856,6 +1900,12 @@ for (;;)
   if (srs.matchnum > 0 && (flags & nf_centre) == 0)
     pn_notelength = mac_muldiv(pn_notelength, srs.matchnum, srs.matchden);
 
+  /* Keep track of the longest and shortest notes/rests. */
+
+  if (pn_notelength > srs.longest_note) srs.longest_note = pn_notelength;
+    else if (pn_notelength < srs.shortest_note)
+      srs.shortest_note = pn_notelength;
+
   /* If a whole bar rest, flag it for centring unless we are in an unchecked
   bar. (If the rest was specified using the ! notation it will already be
   flagged for centring. This code covers other cases.) We must do this before
@@ -1954,7 +2004,7 @@ for (;;)
         case 'm':
         if (pn_chordcount != 0) error(ERR116, "masquerade setting");
         masq = (read_c == 'm')? crotchet : minim;
-        flags &= ~(nf_dot|nf_dot2|nf_plus);
+        dots = 0;
         for (;;)
           {
           read_nextc();
@@ -1968,24 +2018,31 @@ for (;;)
           error_skip(ERR115, '\\');
           masq = MASQ_UNSET;
           }
+
         if (read_c == '.')
           {
-          read_nextc();
-          if (read_c == '+')
+          for (int i = 3; i > 0; i--)
             {
-            read_nextc();
-            flags |= nf_plus;
-            }
-          else
-            {
-            flags |= nf_dot;
-            if (read_c == '.')
+            if (Ustrncmp(main_readbuffer + read_i, "...", i) == 0)
               {
-              read_nextc();
-              flags |= nf_dot2;
+              dots = i + 1;
+              read_i += i;
+              break;
               }
             }
+          if (dots == 0)
+            {
+            if (main_readbuffer[read_i] == '+')
+              {
+              dots = 255;
+              read_i++;
+              }
+            else dots = 1;
+            }
+
+          read_nextc();  /* Next character becomes current. */
           }
+
         break;
 
         case 'n':
@@ -2114,6 +2171,7 @@ for (;;)
             p->x = 0;            /* Default no horizontal movement */
             p->y = 0;            /* Default no vertical movement */
             p->bflags = 0;       /* Default no bracketing */
+            p->troffset = 0;     /* Offset used only for trill */
 
             /* Tremolos have no horizontal movement or bracketing; arpeggios
             and spread have no bracketing, but can have arbitrary movement.
@@ -2144,6 +2202,41 @@ for (;;)
               pf = NULL;
               break;
 
+              /* For trills we have to figure out the trill offset. */
+
+              case or_tr:
+              case or_trsh:
+              case or_trfl:
+              case or_trnat:
+                {
+                int offset = (char_orig == 'b' || char_orig == 'e')? 2 : 4;
+                int nextacc;
+
+                switch(accororn)
+                  {
+                  case or_tr:
+                  nextacc = read_baraccs[pitch_orig + offset];
+                  break;
+
+                  case or_trsh:
+                  nextacc = 2;
+                  break;
+
+                  case or_trfl:
+                  nextacc = -2;
+                  break;
+
+                  default:      /* Avoids compiler warning */
+                  case or_nat:
+                  nextacc = 0;
+                  break;
+                  }
+
+                offset += nextacc - read_baraccs[pitch_orig];
+                p->troffset = offset;
+                }
+              break;
+
               default:
               break;
               }
@@ -2237,11 +2330,11 @@ for (;;)
 
   else if (pn_notelength != chordlength)
     {
-    if (pn_notetype == crotchet && (flags & nf_dotted) == 0)
+    if (pn_notetype == crotchet && dots == 0)
       {
       pn_notelength = chordlength;
       pn_notetype = srs.firstnoteptr->notetype;
-      flags |= srs.firstnoteptr->flags & nf_dotted;
+      dots = srs.firstnoteptr->dots;
       }
     else error(ERR124);
     }
@@ -2288,7 +2381,11 @@ for (;;)
     {
     noteptr->type = b_note;
     srs.firstnoteptr = noteptr;
-    if (pitch != 0) srs.lastnoteptr = noteptr;
+    if (pitch != 0)
+      {
+      srs.lastnoteptr = noteptr;
+      srs.laststemforce = pn_stemforce;
+      }
     }
 
   /* Insert the data for this note, and count it (within a chord). */
@@ -2296,8 +2393,10 @@ for (;;)
   noteptr->notetype = pn_notetype;
   noteptr->masq = (uint8_t)masq;
   noteptr->acc = acc;
+  noteptr->dots = dots;
   noteptr->acc_orig = acc_orig;
   noteptr->char_orig = char_orig;
+  noteptr->dots_orig = dots_orig;
   noteptr->spitch = pitch;
   noteptr->flags = flags;
   noteptr->acflags = acflags;
@@ -2327,8 +2426,8 @@ for (;;)
       noteptr->notetype = pn_notetype;
       noteptr->acflags &= ~af_accents;
       noteptr->length = pn_notelength;
-      noteptr->flags = (noteptr->flags & ~nf_dotted & ~nf_stem) |
-        (flags & (nf_dotted | nf_stem));
+      noteptr->flags = (noteptr->flags & ~nf_stem) | (flags & nf_stem);
+      noteptr->dots = dots;
 
       if (noteptr->spitch > pn_maxpitch) pn_maxpitch = noteptr->spitch;
       if (noteptr->spitch < pn_minpitch) pn_minpitch = noteptr->spitch;
